import logging
#
# from .mixin.caption import CaptionMixin
# from .mixin.channel import ChannelMixin
# from .mixin.comment import CommentThreadsMixin
# from .mixin.search import SearchMixin
#
# __VERSION__ = "0.0.1"
#
# from ytvs.mixin.search import SearchMixin
# from .mixin.video import VideoMixin
#
# DEFAULT_LOGGER = logging.getLogger("youtubeapi")
#
#
# class Client(
#     SearchMixin,
#     VideoMixin,
#     ChannelMixin,
#     CommentThreadsMixin,
#     CaptionMixin
# ):
#     pass
